package Cm;

import java.util.*;
import java.io.*;
import java.net.*;
import java.awt.*;

class Connect extends Thread
{
  class ConnectionLostException extends Exception {};

  public Connect (String name, Socket socket)
      {
        _name = name;
        _socket = socket;
        _message = new Message ();
        _hasToRun = true;

	_connectDebug = (Integer.getInteger ("CMCONNECTDEBUG", 0)).intValue ();
	if (_connectDebug  == 1)
	  {
	    System.out.print ("Connect> name=");
	    System.out.println (name);
	  }

        Server.add_connection (name, this);
        listen ();
      }

  public void set_socket (Socket socket)
      {
        if ((_socket == null) &&
            (socket != null))
          {
            _socket = socket;
          }
      }

  public void set_infos (String name, 
                         String host, 
                         int port, 
                         int multiple, 
                         String owner)
      {
        if (_name.compareTo (name) != 0)
          {
            Server.remove_connection (_name);
            _name = name;
            Server.add_connection (name, this);
          }

        _host     = host;
        _port     = port;
        _multiple = multiple;
        _owner    = owner;
      }

  public Socket get_socket ()
      {
        return (_socket);
      }

  public void listen ()
      {
        if (_socket != null)
          {
            try
              {
                _stream = _socket.getInputStream ();
              }
            catch (Exception e)
              {
                System.out.println ("Connect>Error in getInputStream");
              }

            start ();
          }
      }

  public void run ()
      {
        while (_hasToRun)
          {
            try
              {
                int bytes = _stream.available ();

		if (_connectDebug  > 1)
		  {

		    System.out.print ("run> available = ");
		    System.out.println (bytes);
		  }

                if (bytes == 0)
                  {
                    int b;

                    b = _stream.read ();

                    if (b == -1)
                      {
                        _hasToRun = false;
                        throw new ConnectionLostException ();
                          //System.out.println ("run> Connection lost");
                          // stop ();
                      }

                    _message.write ((byte) b);
                  }
                else
                  {
                    _message.write (_stream);
                  }

		if (_connectDebug > 1)
		    {
			System.out.println ("message received :");
			_message.print ();
		    }
                while (_message.check_after_reception () == Message.MessageOk)
                  {
                      // Will handle the message...
                    String type = _message.get_type ();

		    if (_connectDebug == 1)
		      {
			  System.out.println ("Message type: "+type);
		      }

                    Handler handler = Handler.find (type);

                    if (handler != null)
                      {
			try 
			  {
			    handler.run (_message, _name);
			  }
			catch (Exception e)
			  {
			      System.out.println ("run> Handler error : "+ e);
			      e.printStackTrace ();
			      // _message.print ();
			  }
                      }
                    _message.skip_frame ();

		    System.out.println (" message skipped... ");
                  }
              } 
            catch (ConnectionLostException e)
	      {
		if (_connectDebug == 1)
		  {
		      System.out.println ("Connection lost in name=" + _name);
		  }
                _hasToRun = false;
	      } 
            catch (Exception e)
              {
		if (_connectDebug == 1)
		  {
		    System.out.println ("run> Message error : "+ e);
		    _message.print ();
		     _hasToRun = false;
		  }
              }
          }

        Server.remove_connection (_name);
      }

  private String      _name;
  private Socket      _socket;
  private InputStream _stream;
  private Message   _message;

  private String  _host;
  private int     _port;
  private int     _multiple;
  private String  _owner;
  private boolean _hasToRun;
  private int _connectDebug = 0;
}
